@@ -1,8 +1,8 @@
 /* ===================================================
  * JFreeSVG : an SVG library for the Java(tm) platform
  * ===================================================
- * 
- * (C)opyright 2013-2020, by Object Refinery Limited.  All rights reserved.
+ *
+ * (C)opyright 2013-present, by David Gilbert.  All rights reserved.
  *
  * Project Info:  http://www.jfree.org/jfreesvg/index.html
  * 
@@ -32,59 +32,51 @@
 
 package org.jfree.svg;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-import java.awt.BasicStroke;
-import java.awt.Color;
-import java.awt.Font;
-import java.awt.FontMetrics;
-import java.awt.GradientPaint;
-import java.awt.Graphics2D;
-import java.awt.Image;
-import java.awt.Rectangle;
-import java.awt.RenderingHints;
-import java.awt.Shape;
+import java.awt.*;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Line2D;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
-import org.jfree.svg.SVGGraphics2D;
-import org.jfree.svg.SVGHints;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.BeforeEach;
+
+import static org.junit.jupiter.api.Assertions.*;
 
 /**
  * Some tests for a Graphics2D implementation.  All tests should pass with the
  * Graphics2D instance from a BufferedImage (which we can treat as a reference
  * implementation).
  */
-public class TestSVGGraphics2D {
+class TestSVGGraphics2D {
+    
+    /** 
+     * Change this to true to test against a reference Graphics2D 
+     * implementation from the JDK.  This is useful to verify that the tests
+     * are correct.
+     */
+    private static final boolean TEST_REFERENCE_IMPLEMENTATION = false;
     
     private Graphics2D g2;
     
-    @Before
+    @BeforeEach
     public void setUp() {
-        // to test a reference implementation, use this Graphics2D from a
-        // BufferedImage in the JDK
-        //BufferedImage img = new BufferedImage(10, 20, BufferedImage.TYPE_INT_ARGB);
-        //this.g2 = img.createGraphics();
-        
-        // Test SVGGraphics2D...
-        this.g2 = new SVGGraphics2D(10, 20);
- 
+        if (TEST_REFERENCE_IMPLEMENTATION) {
+            // to test a reference implementation, use this Graphics2D from a
+            // BufferedImage in the JDK
+            BufferedImage img = new BufferedImage(10, 20, BufferedImage.TYPE_INT_ARGB);
+            this.g2 = img.createGraphics();
+        } else {
+            // Test SVGGraphics2D...
+            this.g2 = new SVGGraphics2D(10.0, 20.0);
+        }
     }
     
     /**
      * Checks that the default transform is an identity transform.
      */
     @Test
-    public void checkDefaultTransform() {
+    void checkDefaultTransform() {
         assertEquals(new AffineTransform(), g2.getTransform());
     }
     
@@ -94,7 +86,7 @@ public class TestSVGGraphics2D {
      * should be returning a copy of the actual transform object.
      */
     @Test
-    public void checkGetTransformSafety() {
+    void checkGetTransformSafety() {
         AffineTransform t = g2.getTransform();
         t.rotate(Math.PI);
         assertNotEquals(t, g2.getTransform());
@@ -105,7 +97,7 @@ public class TestSVGGraphics2D {
      * A basic check that setTransform() does indeed update the transform.
      */
     @Test
-    public void setTransform() {
+    void setTransform() {
         AffineTransform t = new AffineTransform(1, 2, 3, 4, 5, 6);
         g2.setTransform(t);
         assertEquals(t, g2.getTransform());
@@ -117,7 +109,7 @@ public class TestSVGGraphics2D {
         // in spite of the docs saying that null is accepted this gives
         // a NullPointerException with SunGraphics2D.
         //g2.setTransform(null);
-        //Assert.assertEquals(new AffineTransform(), g2.getTransform());
+        //assertEquals(new AffineTransform(), g2.getTransform());
     }
     
     /**
@@ -128,7 +120,7 @@ public class TestSVGGraphics2D {
      * AffineTransform.
      */
     @Test
-    public void checkSetTransformSafety() {
+    void checkSetTransformSafety() {
         AffineTransform t = AffineTransform.getTranslateInstance(1.0, 2.0);
         g2.setTransform(t);
         assertEquals(t, g2.getTransform());
@@ -137,7 +129,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkSetNonInvertibleTransform() {
+    void checkSetNonInvertibleTransform() {
         AffineTransform t = AffineTransform.getScaleInstance(0.0, 0.0);
         g2.setTransform(t);
         assertEquals(t, g2.getTransform());
@@ -158,7 +150,7 @@ public class TestSVGGraphics2D {
      * translation.
      */
     @Test
-    public void checkTransform() {
+    void checkTransform() {
         AffineTransform t = new AffineTransform();
         this.g2.setTransform(t);
         this.g2.translate(30, 30);
@@ -175,7 +167,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkTransformNull() {
+    void checkTransformNull() {
         try {
             this.g2.transform(null);
             fail("Expected a NullPointerException.");
@@ -188,7 +180,7 @@ public class TestSVGGraphics2D {
      * Basic checks for the scale(x, y) method.
      */
     @Test
-    public void scale() {
+    void scale() {
         g2.scale(0.5, 2.0);
         assertEquals(AffineTransform.getScaleInstance(0.5, 2.0), 
                 g2.getTransform());
@@ -202,7 +194,7 @@ public class TestSVGGraphics2D {
      * gives the correct values.
      */
     @Test
-    public void translateFollowedByScale() {
+    void translateFollowedByScale() {
         g2.translate(2, 3);
         assertEquals(AffineTransform.getTranslateInstance(2.0, 3.0), 
                 g2.getTransform());
@@ -216,7 +208,7 @@ public class TestSVGGraphics2D {
      * gives the correct values.
      */    
     @Test
-    public void scaleFollowedByTranslate() {
+    void scaleFollowedByTranslate() {
         g2.scale(2, 2);
         assertEquals(AffineTransform.getScaleInstance(2.0, 2.0), 
                 g2.getTransform());
@@ -228,7 +220,7 @@ public class TestSVGGraphics2D {
     private static final double EPSILON = 0.000000001;
     
     @Test
-    public void scaleFollowedByRotate() {
+    void scaleFollowedByRotate() {
         g2.scale(2, 2);
         assertEquals(AffineTransform.getScaleInstance(2.0, 2.0), 
                 g2.getTransform());
@@ -243,7 +235,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void rotateFollowedByScale() {
+    void rotateFollowedByScale() {
         g2.rotate(Math.PI);
         assertEquals(AffineTransform.getRotateInstance(Math.PI), 
                 g2.getTransform());
@@ -258,21 +250,21 @@ public class TestSVGGraphics2D {
      * a new object each time.
      */
     @Test
-    public void checkGetClipSafety() {
+    void checkGetClipSafety() {
         Rectangle2D r = new Rectangle2D.Double(0, 0, 1, 1);
         this.g2.setClip(r);
         Shape s = this.g2.getClip();
-        assertFalse(r == s);
+        assertNotSame(r, s);
         Shape s2 = this.g2.getClip();
-        assertFalse(s == s2);
+        assertNotSame(s, s2);
     }
     
     /**
-     * The default user clip should be <code>null</code>.
+     * The default user clip should be {@code null}.
      */
     @Test
-    public void checkDefaultClip() {
-        assertNull("Default user clip should be null.", g2.getClip());
+    void checkDefaultClip() {
+        assertNull(g2.getClip(), "Default user clip should be null.");
     }
     
     /**
@@ -280,7 +272,7 @@ public class TestSVGGraphics2D {
      * the bounds.
      */
     @Test
-    public void checkGetClipBounds() {
+    void checkGetClipBounds() {
         Rectangle2D r = new Rectangle2D.Double(0.25, 0.25, 0.5, 0.5);
         this.g2.setClip(r);
         assertEquals(new Rectangle(0, 0, 1, 1), this.g2.getClipBounds());       
@@ -291,7 +283,7 @@ public class TestSVGGraphics2D {
      * {@code null}.
      */
     @Test
-    public void checkGetClipBoundsWhenClipIsNull() {
+    void checkGetClipBoundsWhenClipIsNull() {
         this.g2.setClip(null);
         assertNull(this.g2.getClipBounds());
     }
@@ -301,16 +293,20 @@ public class TestSVGGraphics2D {
      * existing clip region.
      */
     @Test
-    public void checkClip() {
+    void checkClip() {
         Rectangle2D r = new Rectangle2D.Double(1.0, 1.0, 3.0, 3.0);
         this.g2.setClip(r);
         this.g2.clip(new Rectangle2D.Double(0.0, 0.0, 2.0, 2.0));
         assertEquals(new Rectangle2D.Double(1.0, 1.0, 1.0, 1.0), 
                 this.g2.getClip().getBounds2D());
     }
-    
+
+    /**
+     * Check that if the user clip is non-intersecting with the existing clip, then
+     * the clip is empty.
+     */
     @Test
-    public void checkNonIntersectingClip() {
+    void checkNonIntersectingClip() {
         Rectangle2D r = new Rectangle2D.Double(1.0, 1.0, 3.0, 3.0);
         this.g2.setClip(r);
         this.g2.clip(new Rectangle2D.Double(5.0, 5.0, 1.0, 1.0));
@@ -322,7 +318,7 @@ public class TestSVGGraphics2D {
      * modified clip.
      */
     @Test
-    public void checkClipAfterScaling() {
+    void checkClipAfterScaling() {
         Rectangle2D r = new Rectangle2D.Double(1, 2, 3, 0.5);
         this.g2.setClip(r);
         assertEquals(r, this.g2.getClip().getBounds2D());
@@ -342,7 +338,7 @@ public class TestSVGGraphics2D {
      * Translating will change the existing clip.
      */
     @Test
-    public void checkClipAfterTranslate() {
+    void checkClipAfterTranslate() {
         Rectangle2D clip = new Rectangle2D.Double(0.0, 0.0, 1.0, 1.0);
         this.g2.setClip(clip);
         assertEquals(clip, this.g2.getClip().getBounds2D());
@@ -352,7 +348,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkSetClipAfterTranslate() {
+    void checkSetClipAfterTranslate() {
         this.g2.translate(1.0, 2.0);
         this.g2.setClip(0, 0, 1, 1);
         assertEquals(new Rectangle(0, 0, 1, 1), this.g2.getClip().getBounds());
@@ -364,7 +360,7 @@ public class TestSVGGraphics2D {
      * Transforming will change the reported clipping shape.
      */
     @Test
-    public void checkClipAfterTransform() {
+    void checkClipAfterTransform() {
         Rectangle2D clip = new Rectangle2D.Double(0, 0, 1, 1);
         this.g2.setClip(clip);
         assertEquals(clip, this.g2.getClip().getBounds2D());
@@ -381,24 +377,41 @@ public class TestSVGGraphics2D {
      * Clipping with a line makes no sense, but the API allows it so we should
      * not fail.  In fact, running with a JDK Graphics2D (from a BufferedImage)
      * it seems that the bounding rectangle of the line is used for clipping...
-     * does that make sense?  Switching off the test for now.
+     * does that make sense?  Matching the behaviour for now.
      */
     @Test
-    @Ignore
-    public void checkClipWithLine2D() {
+    void checkClipWithLine2D() {
         Rectangle2D r = new Rectangle2D.Double(1.0, 1.0, 3.0, 3.0);
         this.g2.setClip(r);
         this.g2.clip(new Line2D.Double(1.0, 2.0, 3.0, 4.0));
-        //assertEquals(new Rectangle2D.Double(1.0, 2.0, 2.0, 2.0), 
-        //        this.g2.getClip().getBounds2D());
+        assertEquals(new Rectangle2D.Double(1.0, 2.0, 2.0, 2.0), 
+                this.g2.getClip().getBounds2D());
         //assertTrue(this.g2.getClip().getBounds2D().isEmpty());        
     }
     
+    /**
+     * Clipping with a null argument is "not recommended" according to the 
+     * latest API docs (https://bugs.java.com/bugdatabase/view_bug.do?bug_id=6206189).
+     */
+    @Test
+    void checkClipWithNullArgument() {
+        // when there is a current clip set, a null pointer exception is expected
+        this.g2.setClip(new Rectangle2D.Double(1.0, 2.0, 3.0, 4.0));
+        assertThrows(NullPointerException.class, () -> this.g2.clip(null));
+        
+        this.g2.setClip(null);
+        try {
+            this.g2.clip(null);
+        } catch (Exception e) {
+            fail("No exception expected.");             
+        }
+    }
+    
     /**
      * A simple check for a call to clipRect().
      */
     @Test
-    public void checkClipRect() {
+    void checkClipRect() {
         Rectangle2D clip = new Rectangle2D.Double(0, 0, 5, 5);
         this.g2.setClip(clip);
         
@@ -408,7 +421,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkClipRectParams() {
+    void checkClipRectParams() {
         Rectangle2D clip = new Rectangle2D.Double(0, 0, 5, 5);
         this.g2.setClip(clip);
         
@@ -421,9 +434,9 @@ public class TestSVGGraphics2D {
         this.g2.clipRect(2, 1, 4, -2);
         assertTrue(this.g2.getClip().getBounds2D().isEmpty());    
     }
-    
+
     @Test
-    public void checkDrawStringWithNullString() {
+    void checkDrawStringWithNullString() {
         try {
             g2.drawString((String) null, 1, 2);
             fail("There should be a NullPointerException.");
@@ -439,7 +452,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkDrawStringWithEmptyString() {
+    void checkDrawStringWithEmptyString() {
         // this should not cause any exception 
         g2.setRenderingHint(SVGHints.KEY_DRAW_STRING_TYPE, SVGHints.VALUE_DRAW_STRING_TYPE_VECTOR);
         g2.drawString("", 1, 2);
@@ -450,8 +463,8 @@ public class TestSVGGraphics2D {
     /**
      * Some checks for the create() method.
      */
-    @Test 
-    public void checkCreate() {
+    @Test
+    void checkCreate() {
         this.g2.setClip(new Rectangle(1, 2, 3, 4));
         Graphics2D copy = (Graphics2D) g2.create();
         assertEquals(copy.getBackground(), g2.getBackground());
@@ -474,13 +487,13 @@ public class TestSVGGraphics2D {
      * Paint instance.  
      */
     @Test
-    public void checkSetPaintSafety() {
+    void checkSetPaintSafety() {
         Point2D pt1 = new Point2D.Double(1.0, 2.0);
         Point2D pt2 = new Point2D.Double(3.0, 4.0);
         GradientPaint gp = new GradientPaint(pt1, Color.RED, pt2, Color.BLUE);
         this.g2.setPaint(gp);
         assertEquals(gp, this.g2.getPaint());
-        assertTrue(gp == this.g2.getPaint());
+        assertSame(gp, this.g2.getPaint());
         pt1.setLocation(7.0, 7.0);
         assertEquals(gp, this.g2.getPaint());
     }
@@ -490,7 +503,7 @@ public class TestSVGGraphics2D {
      * impact on the current paint (that is, the call is silently ignored).
      */
     @Test
-    public void checkSetPaintNull() {
+    void checkSetPaintNull() {
         this.g2.setPaint(Color.RED);
         // this next call should have no impact
         this.g2.setPaint(null);
@@ -502,7 +515,7 @@ public class TestSVGGraphics2D {
      * background color.
      */
     @Test
-    public void checkSetPaintAlsoUpdatesColorButNotBackground() {
+    void checkSetPaintAlsoUpdatesColorButNotBackground() {
         Color existingBackground = this.g2.getBackground();
         this.g2.setPaint(Color.MAGENTA);
         assertEquals(Color.MAGENTA, this.g2.getPaint());
@@ -510,8 +523,28 @@ public class TestSVGGraphics2D {
         assertEquals(existingBackground, this.g2.getBackground());
     }
     
+    /**
+     * If setPaint() is called with an argument that is not an instance of
+     * Color, then the existing color remains unchanged.
+     */
     @Test
-    public void checkSetColorAlsoUpdatesPaint() {
+    void checkSetPaintDoesNotUpdateColor() {
+        GradientPaint gp = new GradientPaint(1.0f, 2.0f, Color.RED, 
+                3.0f, 4.0f, Color.BLUE);
+        this.g2.setColor(Color.MAGENTA);
+        this.g2.setPaint(gp);
+        assertEquals(gp, this.g2.getPaint());
+        assertEquals(Color.MAGENTA, this.g2.getColor());
+    }    
+    
+    /**
+     * Verifies that setting the old AWT color attribute also updates the
+     * Java2D paint attribute.
+     * 
+     * @see #checkSetPaintAlsoUpdatesColorButNotBackground() 
+     */
+    @Test
+    void checkSetColorAlsoUpdatesPaint() {
         this.g2.setColor(Color.MAGENTA);
         assertEquals(Color.MAGENTA, this.g2.getPaint());
         assertEquals(Color.MAGENTA, this.g2.getColor());
@@ -523,7 +556,7 @@ public class TestSVGGraphics2D {
      * setPaint() method.
      */
     @Test
-    public void checkSetColorNull() {
+    void checkSetColorNull() {
         this.g2.setColor(Color.RED);
         this.g2.setColor(null);
         assertEquals(Color.RED, this.g2.getColor());
@@ -533,11 +566,11 @@ public class TestSVGGraphics2D {
      * Setting the background color does not change the color or paint.
      */
     @Test
-    public void checkSetBackground() {
+    void checkSetBackground() {
         this.g2.setBackground(Color.CYAN);
         assertEquals(Color.CYAN, this.g2.getBackground());
-        assertFalse(Color.CYAN.equals(this.g2.getColor()));
-        assertFalse(Color.CYAN.equals(this.g2.getPaint()));
+        assertNotEquals(Color.CYAN, this.g2.getColor());
+        assertNotEquals(Color.CYAN, this.g2.getPaint());
     }
 
     /**
@@ -545,10 +578,10 @@ public class TestSVGGraphics2D {
      * allowing null (this is inconsistent with the behaviour of setColor()).
      */
     @Test
-    public void checkSetBackgroundNull() {
+    void checkSetBackgroundNull() {
         this.g2.setBackground(Color.RED);
         this.g2.setBackground(null);
-        assertEquals(null, this.g2.getBackground());
+        assertNull(this.g2.getBackground());
     }
     
     /**
@@ -558,7 +591,7 @@ public class TestSVGGraphics2D {
      * is no easy way to test for that.
      */
     @Test
-    public void checkClearRectWithNullBackground() {
+    void checkClearRectWithNullBackground() {
         this.g2.setBackground(null);
         this.g2.clearRect(1, 2, 3, 4);
         //no exceptions and we're good
@@ -569,7 +602,7 @@ public class TestSVGGraphics2D {
      * observed to throw an IllegalArgumentException.
      */
     @Test
-    public void checkSetCompositeNull() {
+    void checkSetCompositeNull() {
         try {
             this.g2.setComposite(null);
             fail("Expected an IllegalArgumentException.");
@@ -579,7 +612,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkSetStrokeNull() {
+    void checkSetStrokeNull() {
         try {
             this.g2.setStroke(null);
             fail("Expected an IllegalArgumentException.");
@@ -592,7 +625,7 @@ public class TestSVGGraphics2D {
      * Basic check of set then get.
      */
     @Test
-    public void checkSetRenderingHint() {
+    void checkSetRenderingHint() {
         this.g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, 
                 RenderingHints.VALUE_STROKE_PURE);
         assertEquals(RenderingHints.VALUE_STROKE_PURE, 
@@ -604,7 +637,7 @@ public class TestSVGGraphics2D {
      * NullPointerException when the key is null.
      */
     @Test
-    public void checkSetRenderingHintWithNullKey() {
+    void checkSetRenderingHintWithNullKey() {
         try {
             this.g2.setRenderingHint(null, "XYZ");
             fail("NullPointerException is expected here.");
@@ -618,7 +651,7 @@ public class TestSVGGraphics2D {
      * and return null in that case.
      */
     @Test
-    public void checkGetRenderingHintWithNullKey() {
+    void checkGetRenderingHintWithNullKey() {
         assertNull(this.g2.getRenderingHint(null));
     }
     
@@ -626,7 +659,7 @@ public class TestSVGGraphics2D {
      * Check setting a hint with a value that doesn't match the key.
      */
     @Test
-    public void checkSetRenderingHintWithInconsistentValue() {
+    void checkSetRenderingHintWithInconsistentValue() {
         try {
             this.g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, 
                     RenderingHints.VALUE_ANTIALIAS_DEFAULT);
@@ -641,7 +674,7 @@ public class TestSVGGraphics2D {
      * changing it will not affect the state of the Graphics2D instance.
      */
     @Test
-    public void checkGetRenderingHintsSafety() {
+    void checkGetRenderingHintsSafety() {
         this.g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
                 RenderingHints.VALUE_ANTIALIAS_OFF);
         RenderingHints hints = this.g2.getRenderingHints();
@@ -652,7 +685,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkSetRenderingHintsNull() {
+    void checkSetRenderingHintsNull() {
         try {
             this.g2.setRenderingHints(null);
             fail("NullPointerException expected.");
@@ -662,7 +695,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkHit() {
+    void checkHit() {
         Shape shape = new Rectangle2D.Double(0.0, 0.0, 1.0, 1.0);
         Rectangle r = new Rectangle(2, 2, 2, 2);
         assertFalse(this.g2.hit(r, shape, false));
@@ -671,7 +704,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkHitForOutline() {
+    void checkHitForOutline() {
         Shape shape = new Rectangle2D.Double(0.0, 0.0, 3.0, 3.0);
         Rectangle r = new Rectangle(1, 1, 1, 1);
         assertFalse(this.g2.hit(r, shape, true));
@@ -686,7 +719,7 @@ public class TestSVGGraphics2D {
      * to null does not change the current font setting.
      */
     @Test
-    public void checkSetFontNull() {
+    void checkSetFontNull() {
         Font f = new Font("Serif", Font.PLAIN, 8);
         this.g2.setFont(f);
         assertEquals(f, this.g2.getFont());
@@ -695,7 +728,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void checkDefaultStroke() {
+    void checkDefaultStroke() {
         BasicStroke s = (BasicStroke) this.g2.getStroke();
         assertEquals(BasicStroke.CAP_SQUARE, s.getEndCap());
         assertEquals(1.0f, s.getLineWidth(), EPSILON);
@@ -703,10 +736,10 @@ public class TestSVGGraphics2D {
     }
     
     /**
-     * Check that a null GlyphVector throws a <code>NullPointerException</code>.
+     * Check that a null GlyphVector throws a {@code NullPointerException}.
      */
     @Test
-    public void drawGlyphVectorNull() {
+    void drawGlyphVectorNull() {
         try {
             g2.drawGlyphVector(null, 10, 10);
             fail("Expecting a NullPointerException.");
@@ -719,7 +752,7 @@ public class TestSVGGraphics2D {
      * Check the shear() method.
      */
     @Test
-    public void shear() {
+    void shear() {
         g2.setTransform(new AffineTransform());
         g2.shear(2.0, 3.0);
         assertEquals(new AffineTransform(1, 3, 2, 1, 0, 0), g2.getTransform());
@@ -729,7 +762,7 @@ public class TestSVGGraphics2D {
      * Checks a translate() followed by a shear().
      */
     @Test
-    public void shearFollowingTranslate() {
+    void shearFollowingTranslate() {
         g2.setTransform(new AffineTransform());
         g2.translate(10.0, 20.0);
         g2.shear(2.0, 3.0);
@@ -737,7 +770,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void drawImageWithNullBackground() {
+    void drawImageWithNullBackground() {
         Image img = new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
         g2.drawImage(img, 10, 10, null, null);
         assertTrue(true); // won't get here if there's an exception above
@@ -747,14 +780,14 @@ public class TestSVGGraphics2D {
      * https://github.com/jfree/jfreesvg/issues/6
      */
     @Test
-    public void drawImageWithNullTransform() {
+    void drawImageWithNullTransform() {
         Image img = new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
         g2.drawImage(img, null, null);
         assertTrue(true); // won't get here if there's an exception above
     }
     
     @Test
-    public void drawImageWithNullImage() {
+    void drawImageWithNullImage() {
         // API docs say method does nothing if img is null
         // still seems to return true
         assertTrue(g2.drawImage(null, 10, 20, null));
@@ -766,18 +799,65 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void drawImageWithNegativeDimensions() {
+    void drawImageWithNegativeDimensions() {
         Image img = new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
         assertTrue(g2.drawImage(img, 1, 2, -10, 10, null));
         assertTrue(g2.drawImage(img, 1, 2, 10, -10, null)); 
     }
 
-    /** 
+    /**
+     * Check that the color is not changed by setting a clip.  In some
+     * implementations the clip is saved/restored as part of the overall
+     * graphics state so clipping can impact other attributes.
+     */
+    @Test
+    void checkColorAfterSetClip() {
+        this.g2.setColor(Color.RED);
+        assertEquals(Color.RED, this.g2.getColor());
+        this.g2.setClip(0, 0, 10, 10);
+        assertEquals(Color.RED, this.g2.getColor());
+        this.g2.setColor(Color.BLUE);
+        assertEquals(Color.BLUE, this.g2.getColor());
+        this.g2.setClip(0, 0, 20, 20);
+        assertEquals(Color.BLUE, this.g2.getColor());
+    }
+
+    /**
+     * See https://github.com/jfree/fxgraphics2d/issues/6
+     */
+    @Test
+    void checkFontAfterSetClip() {
+        this.g2.setFont(new Font(Font.DIALOG, Font.BOLD, 12));
+        assertEquals(new Font(Font.DIALOG, Font.BOLD, 12), this.g2.getFont());
+        this.g2.setClip(0, 0, 10, 10);
+        assertEquals(new Font(Font.DIALOG, Font.BOLD, 12), this.g2.getFont());
+        this.g2.setFont(new Font(Font.DIALOG, Font.BOLD, 24));
+        assertEquals(new Font(Font.DIALOG, Font.BOLD, 24), this.g2.getFont());
+        this.g2.setClip(0, 0, 20, 20);
+        assertEquals(new Font(Font.DIALOG, Font.BOLD, 24), this.g2.getFont());
+    }
+
+    /**
+     * See https://github.com/jfree/fxgraphics2d/issues/6
+     */
+    @Test
+    void checkStrokeAfterSetClip() {
+        this.g2.setStroke(new BasicStroke(1.0f));
+        assertEquals(new BasicStroke(1.0f), this.g2.getStroke());
+        this.g2.setClip(0, 0, 10, 10);
+        assertEquals(new BasicStroke(1.0f), this.g2.getStroke());
+        this.g2.setStroke(new BasicStroke(2.0f));
+        assertEquals(new BasicStroke(2.0f), this.g2.getStroke());
+        this.g2.setClip(0, 0, 20, 20);
+        assertEquals(new BasicStroke(2.0f), this.g2.getStroke());
+    }
+
+    /**
      * A test to check whether setting a transform on the Graphics2D affects
      * the results of text measurements performed via getFontMetrics().
      */
     @Test
-    public void testGetFontMetrics() {
+    void testGetFontMetrics() {
         Font f = new Font(Font.SANS_SERIF, Font.PLAIN, 10);
         FontMetrics fm = this.g2.getFontMetrics(f);
         int w = fm.stringWidth("ABC");
@@ -791,7 +871,7 @@ public class TestSVGGraphics2D {
     }
     
     @Test
-    public void drawImageWithNullImageOp() {
+    void drawImageWithNullImageOp() {
         BufferedImage img = new BufferedImage(10, 10, BufferedImage.TYPE_INT_ARGB);
         g2.drawImage(img, null, 2, 3);
         assertTrue(true); // won't get here if there's an exception above        
@@ -800,9 +880,91 @@ public class TestSVGGraphics2D {
     /**
      * API docs say the method does nothing when called with a null image.
      */
-    @Test 
-    public void drawRenderedImageWithNullImage() {
+    @Test
+    void drawRenderedImageWithNullImage() {
         g2.drawRenderedImage(null, AffineTransform.getTranslateInstance(0, 0));
         assertTrue(true); // won't get here if there's an exception above                
     }
+
+    /**
+     * Filling and/or stroking a Rectangle2D with a negative width will not display anything but
+     * should not throw an exception.
+     */
+    @Test
+    void fillOrStrokeRectangleWithNegativeWidthMustNotFail() {
+        g2.draw(new Rectangle2D.Double(0, 0, 0, 10));
+        g2.draw(new Rectangle2D.Double(0, 0, -10, 10));
+        g2.fill(new Rectangle2D.Double(0, 0, 0, 10));
+        g2.fill(new Rectangle2D.Double(0, 0, -10, 10));
+        assertTrue(true); // won't get here if there's an exception above
+    }
+
+    /**
+     * Filling and/or stroking a Rectangle2D with a negative height will not display anything but
+     * should not throw an exception.
+     */
+    @Test
+    void fillOrStrokeRectangleWithNegativeHeightMustNotFail() {
+        g2.draw(new Rectangle2D.Double(0, 0, 0, 10));
+        g2.draw(new Rectangle2D.Double(0, 0, 10, -10));
+        g2.fill(new Rectangle2D.Double(0, 0, 0, 10));
+        g2.fill(new Rectangle2D.Double(0, 0, 10, -10));
+        assertTrue(true); // won't get here if there's an exception above
+    }
+
+    @Test
+    void checkClipAfterCreate() {
+        this.g2.setClip(10, 20, 30, 40);
+        assertEquals(new Rectangle(10, 20, 30, 40), g2.getClip().getBounds2D());
+
+        Graphics2D g2copy = (Graphics2D) this.g2.create();
+        g2copy.clipRect(11, 21, 10, 10);
+        assertEquals(new Rectangle(11, 21, 10, 10), g2copy.getClip().getBounds2D());
+        g2copy.dispose();
+        assertEquals(new Rectangle(10, 20, 30, 40), g2.getClip().getBounds2D());
+    }
+
+    @Test
+    void checkGradientPaintRefGeneration() {
+        if (!(this.g2 instanceof SVGGraphics2D)) return;
+        SVGGraphics2D svg2 = (SVGGraphics2D) this.g2;
+        GradientPaint gp0 = new GradientPaint(1.0f, 2.0f, Color.RED, 3.0f, 4.0f, Color.BLUE);
+        svg2.setPaint(gp0);
+        assertEquals(svg2.defsKeyPrefix + "gp0", svg2.gradientPaintRef);
+        GradientPaint gp1 = new GradientPaint(1.0f, 2.0f, Color.YELLOW, 3.0f, 4.0f, Color.GREEN);
+        svg2.setPaint(gp1);
+        assertEquals(svg2.defsKeyPrefix + "gp1", svg2.gradientPaintRef);
+        svg2.setPaint(gp0);
+        assertEquals(svg2.defsKeyPrefix + "gp0", svg2.gradientPaintRef);
+    }
+
+    @Test
+    void checkLinearGradientPaintRefGeneration() {
+        if (!(this.g2 instanceof SVGGraphics2D)) return;
+        SVGGraphics2D svg2 = (SVGGraphics2D) this.g2;
+        var lgp0 = new LinearGradientPaint(1.0f, 2.0f,3.0f, 4.0f, new float[] { 0.0f, 0.5f, 1.0f }, new Color[] { Color.RED, Color.BLUE, Color.GREEN });
+        svg2.setPaint(lgp0);
+        assertEquals(svg2.defsKeyPrefix + "lgp0", svg2.gradientPaintRef);
+        var lgp1 = new LinearGradientPaint(1.0f, 2.0f,3.0f, 4.0f, new float[] { 0.0f, 0.5f, 1.0f }, new Color[] { Color.YELLOW, Color.CYAN, Color.GRAY });
+        svg2.setPaint(lgp1);
+        assertEquals(svg2.defsKeyPrefix + "lgp1", svg2.gradientPaintRef);
+        svg2.setPaint(lgp0);
+        assertEquals(svg2.defsKeyPrefix + "lgp0", svg2.gradientPaintRef);
+    }
+
+    @Test
+    void checkRadialGradientPaintRefGeneration() {
+        if (!(this.g2 instanceof SVGGraphics2D)) return;
+        SVGGraphics2D svg2 = (SVGGraphics2D) this.g2;
+        var rgp0 = new RadialGradientPaint(1.0f, 2.0f, 3.0f, new float[] { 0.0f, 0.5f, 1.0f },
+                new Color[] { Color.RED, Color.BLUE, Color.GREEN });
+        svg2.setPaint(rgp0);
+        assertEquals(svg2.defsKeyPrefix + "rgp0", svg2.gradientPaintRef);
+        var rgp1 = new RadialGradientPaint(1.0f, 2.0f, 4.0f, new float[] { 0.0f, 0.5f, 1.0f },
+                new Color[] { Color.YELLOW, Color.CYAN, Color.GRAY });
+        svg2.setPaint(rgp1);
+        assertEquals(svg2.defsKeyPrefix + "rgp1", svg2.gradientPaintRef);
+        svg2.setPaint(rgp0);
+        assertEquals(svg2.defsKeyPrefix + "rgp0", svg2.gradientPaintRef);
+    }
 }
